/********************************************************************
* FIG4.G :
*
* PROGRAM TO GRAPH Response to oil shock for various measures of oil
*       with 7 and 12 monthly lags used for estimation
*
* Only graphs: nominal price level, hp, mork, hamilton
*
* Last program update: 4/29/97
********************************************************************/

#lineson;
new;
library pgraph;
graphset;
_pdate ="";


/********************************************************************
* INPUT DATA AND SETUP
********************************************************************/
nsteps=48;
ymx = .1;
ymn = -.15;
ystp = .05;
pmx = .1;
pmn = -.1;
pstp = .05;
fmx = 20;
fmn = -15;
fstp = 35/4;

load data4_7[nsteps,2] = 7lags\f2o2_y.dta;
load data10_7[nsteps,2] = 7lags\f2o4_y.dta;
load data13_7[nsteps,2] =7lags\f2o5_y.dta;
load data19_7[nsteps,2] = 7lags\f2o7_y.dta;

data4_7 = 1000.*data4_7;
data10_7 = 1000.*data10_7;
data13_7 = 1000.*data13_7;
data19_7 = 1000.*data19_7;


load data4_12[nsteps,2] = 12lags\f2o2_y.dta;
load data10_12[nsteps,2] = 12lags\f2o4_y.dta;
load data13_12[nsteps,2] = 12lags\f2o5_y.dta;
load data19_12[nsteps,2] = 12lags\f2o7_y.dta;

data4_12 = 1000.*data4_12;
data10_12 = 1000.*data10_12;
data13_12 = 1000.*data13_12;
data19_12 = 1000.*data19_12;


load data4_16[nsteps,2] = 
16lags\f2o2_y.dta;
load data10_16[nsteps,2] = 16lags\f2o4_y.dta;
load data13_16[nsteps,2] = 
16lags\f2o5_y.dta;
load data19_16[nsteps,2] = 
16lags\f2o7_y.dta;

data4_16 = 1000.*data4_16;
data10_16 = 1000.*data10_16;
data13_16 = 1000.*data13_16;
data19_16 = 1000.*data19_16;

/* Order is mean and standard deviation by column */
/* Order of oil variables is:
1= Log of Relative Price
2= Hamilton
3= Change if log of relative price
4= Change in log of nominal price
5= Mork
6= Rotemberg-Woodford
7= Hoover-Perez
8 = Nordhaus (using Hamilton oil)
*/

output file = junk reset;
output on;

/********************************************************************
* GRAPH
********************************************************************/
graphset;
begwind;
_ptek="Fig8.tkf";

xsize = (8.75/3) - .6;
ysize = (6.4/4)-.25;
marg = 1.5;
botmarg = 1.0;
axhght = 0.36;
titht = .38;
axht = .35;

makewind(xsize,ysize,marg,botmarg+3*ysize,1);
makewind(xsize,ysize,marg+xsize,botmarg+3*ysize,1);
makewind(xsize,ysize,marg+2*xsize,botmarg+3*ysize,1);

makewind(xsize,ysize,marg,botmarg+2*ysize,1);
makewind(xsize,ysize,marg+xsize,botmarg+2*ysize,1);
makewind(xsize,ysize,marg+2*xsize,botmarg+2*ysize,1);

makewind(xsize,ysize,marg,botmarg+ysize,1);
makewind(xsize,ysize,marg+xsize,botmarg+ysize,1);
makewind(xsize,ysize,marg+2*xsize,botmarg+ysize,1);

makewind(xsize,ysize,marg,botmarg,1);
makewind(xsize,ysize,marg+xsize,botmarg,1);
makewind(xsize,ysize,marg+2*xsize,botmarg,1);

x = seqa(0,1,nsteps);
z = zeros(nsteps,1);

setwind(1);
_pcolor=12;
fonts("Microb");
_pmsgctl = 3.5 ~ 7.5 ~ .5 ~ 0 ~ 2 ~ 1 ~ 2;
_pmsgctl = _pmsgctl | (-0.5 ~ 2 ~ 0.5 ~ 90 ~ 2 ~ 1 ~ 2);
_pmsgstr = "7 lags\000Nominal";
margin(.1,0,0,0);
_paxht = axhght;
_ptitlht = titht;
_pnumht = axhght;
_pnum = {1,2,0};
_pltype = (6|1|1|6);
_plwidth = (3|3|3|0);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data10_7[.,1]~(data10_7[.,1]+data10_7[.,2])~
(data10_7[.,1]-data10_7[.,2])~z);

setwind(2);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_pmsgctl = 3.5 ~ 7.5 ~ .5 ~ 0 ~ 2 ~ 1 ~ 2;
_pmsgstr = "12 lags";
_pnum = {1,2,0};
_ptitlht = titht;
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data10_12[.,1]~(data10_12[.,1]+data10_12[.,2])~
(data10_12[.,1]-data10_12[.,2])~z);

setwind(3);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_pmsgctl = 3.5 ~ 7.5 ~ .5 ~ 0 ~ 2 ~ 1 ~ 2;
_pmsgstr="16 lags";
_pnum = {1,2,0};
_ptitlht = titht;
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data10_12[.,1]~(data10_16[.,1]+data10_16[.,2])~
(data10_16[.,1]-data10_16[.,2])~z);

setwind(4);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_pmsgctl =  (-0.5 ~ 2.5 ~ 0.5 ~ 90 ~ 2 ~ 1 ~ 2);
_pmsgstr = "Hoover";

_pnum = {1,2,0};
_ptitlht = titht;
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data19_7[.,1]~(data19_7[.,1]+data19_7[.,2])~
(data19_7[.,1]-data19_7[.,2])~z);

setwind(5);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_pnum = {1,2,0};
_ptitlht = titht;
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data19_12[.,1]~(data19_12[.,1]+data19_12[.,2])~
(data19_12[.,1]-data19_12[.,2])~z);

setwind(6);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data19_16[.,1]~(data19_16[.,1]+data19_16[.,2])~
(data19_16[.,1]-data19_16[.,2])~z);

setwind(7);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_pmsgctl =  (-0.5 ~ 3 ~ 0.5 ~ 90 ~ 2 ~ 1 ~ 2);
_pmsgstr = "Mork";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data13_7[.,1]~(data13_7[.,1]+data13_7[.,2])~
(data13_7[.,1]-data13_7[.,2])~z);

setwind(8);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data13_12[.,1]~(data13_12[.,1]+data13_12[.,2])~
(data13_12[.,1]-data13_12[.,2])~z);

setwind(9);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data13_16[.,1]~(data13_16[.,1]+data13_16[.,2])~
(data13_16[.,1]-data13_16[.,2])~z);

setwind(10);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_pmsgctl =  (-0.5 ~ 2 ~ 0.5 ~ 90 ~ 2 ~ 1 ~ 2);
_pmsgstr = "Hamilton";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data4_7[.,1]~(data4_7[.,1]+data4_7[.,2])~
(data4_7[.,1]-data4_7[.,2])~z);


setwind(11);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data4_12[.,1]~(data4_12[.,1]+data4_12[.,2])~
(data4_12[.,1]-data4_12[.,2])~z);

setwind(12);
graphset;
_pcolor=12;
fonts("Microb");
_ptek="Fig8.tkf";
_ptitlht = titht;
_pnum = {1,2,0};
_plwidth = (3|3|3|0);
_paxht = axhght;
_pnumht = axhght;
_pltype = (6|1|1|6);
xtics(0,nsteps,12,0);
ytics(-1.5,1,.5,0);
xy(x,data4_16[.,1]~(data4_16[.,1]+data4_16[.,2])~
(data4_16[.,1]-data4_16[.,2])~z);

endwind;

output off;
end;

