@ These GAUSS procs are used to bootstrap a VAR @

/* =============================================================== */
proc booty(ystart,bhat,ehat,plag,nburn);
@ inputs
	ystart = (plag x ny) matrix of observations used to start the simulation
	bhat = (ny x nx) matrix of VAR coefficients
		code assumes that nx = 1 + plag*ny
	ehat = (nobs x ny) matrix of VAR residuals
	plag = number of lags in VAR
	nburn = number of initial observations thrown out in order to handle initial conditions
    global
	capK = maximum number of lags considered 
    output
	ystar = (nburn + nobs + capK x ny) matrix of simulated observations  @


local  ny, nx, nobs, nstar, it, ylag,ip,iy,ystar,ustar;

ny = cols(ehat);
nx = cols(bhat);
nobs = rows(ehat);
nstar = nobs + capK;

if nx /= 1 + plag*ny;
	"code assumes standard VAR in which each equation has 1 + ";;
	plag;;"*";;ny;;"variables";
	end;
endif;
ystar = zeros(nburn + nstar,ny);

@ generate a random vector, each of whose elements is equally likely to be one of the 
     integers {1,2,...,nob} @
ustar = 1+trunc(nobs*rndus(nburn+nstar,1,seed1));   
	
ystar[1:plag,.] = ystart[1:plag,.];
it = plag + 1;
do until it > nburn + nstar;

	@ 	 put constant term and plag most recent observations in the 
		[1 x (1 + ny*plag)] vector ylag @
	ylag = multlags(ystar[it-plag:it-1,.]|zeros(1,ny),plag,plag+1);

	@ 	 use VAR coefficients and residuals to calculate value for ystar @
	ystar[it,.] = (bhat*ylag')';
	ystar[it,.] = ystar[it,.] + ehat[ustar[it,1],.];
it = it + 1;
endo;
retp(ystar);
endp;

/* ===================================================================== */
proc impulse(bhat,plag,horiz,iy);
@ inputs:
	bhat =  [ny x (1 + plag*ny)] matrix of VAR coefficients
	plag = number of lags included in estimated VAR
	horiz = number of impulses desired
	iy = variable to be shocked 
   output:
	shockly = [(horiz + 1) x ny] matrix of responses of each of the ny variables for up
		to horizon @
local shockly, ylag, ih,ny,nx;
ny = rows(bhat); 
nx = cols(bhat);
if nx /= 1 + plag*ny;
	"code assumes that each equation in VAR should have 1 plus";;plag;;"times";;
	ny;;"variables";
	end;
endif;
shockly = zeros(horiz+plag,ny);
shockly[plag,iy] = 1;
ih = plag + 1;
do until ih > horiz + plag;
	ylag = multlags(shockly[ih-plag:ih-1,.]|zeros(1,ny),plag,plag+1);	
	ylag[1,1] = 0;
	shockly[ih,.] = (bhat*ylag')';
ih = ih+1;
endo;
retp(shockly[plag:horiz+plag,.]);
endp;


/* ====================================================================== */
proc critic(pval,nchi);
@ this proc finds a value critval such that the probability that a chi-squared variable with nchi
    degrees of freedom exceeds critval is less than pval; use nchi = ny^2 for VAR application @
local critval, it,thresh,try,guess,see;
thresh = 100;
guess = 0;
do until thresh < .01;
	try = seqa(guess,thresh,20);
	see = cdfchic(try,nchi);
	see = abs(see - pval);
	see = try ~ see;
	see = sortc(see,2);
	guess = maxc(see[1,1] - thresh | 0);
thresh = thresh/10;
endo;
retp(guess);
endp;
	

	
