@ This GAUSS code controls bootstrapping analysis @
	output file=junk reset;
	library pgraph;
	graphset;
	_pdate = 0;
format /m1;

/* ==================================================
	load data
  ==================================================== */
nt = 372;
load y[nt,9] = ..\datajdh.txt;
@ first row 1965:1  last row 1995:12 
 	col (1)  output growth
	col (2)  log price level
	col (3)  log commodity price
	col (4)  Hamilton net oil
	col (5)  fed funds rate
	col (6) short rate
	col (7) long rate  @

/* =======================================================
	set global variables used in procs
    ======================================================= */
capK = 24;        @ capK is the maximum number of lags entertained 
			      used capK = 12 for simple analysis and capK =24 for extended @
nfest = capK +1;   @ nfest is the first observation used for estimation (after lags) @
nfirst = nfest;
plag = 7;       @ plag is number of lags in VAR @
nuse = nt - nfest +1;   @ nuse is the number of observations used @
vhat = 0;         @ this will be set to the structural variance-covariance matrix for VAR @
ks = 48;            @ ks is number of horizons used for forecasts or impulse-response @
ny = 7;            @ ny is number of left-hand variables in VAR @
kc = 1;             @ kc = 2 prints output of VAR analysis, kc = 1 suppresses output @
phi = 0;             @ phi will contain lagged VAR coefficients; set to 0 to load procs @
b0 = 0;
seed1 = 3937841;

#include ..\varproc2.g;   @ this adds GAUSS VAR procs @
#include booproc2.g;   @ this adds the proc for bootstrap routines @


crit = critic(.005,ny^2);         @ calculates critical value to use for likelihood ratio tests @


/* =============================================
	estimate an unrestricted VAR with plag lags */

plag = 7;
yq = y[.,1:7];
y2 = multlags(y[.,1:7],plag,nfest);

ystart = yq[nfest-plag:nfest-1,.];    @ will be used for starting values @
yq = yq[nfest:nt,.];

{bhat,vhat,ehat} = varit(yq,y2);
"original var coefficients";
format /m3;
bhat;


/* =======================================================
 generate bootstrap samples 
  ========================================================*/
nmonte = 10000;    @ nmonte is number of bootstrap simulations @
nburn = 100;   @ nburn is number of simulations burned @
nfirst = capK +1;   @ determines which observation is #1 for estimation @
meanb = zeros(ny*(1 + ny*plag),1);
meanb2 = meanb;
likerat = zeros(nmonte,1);


imonte = 1;
do until imonte > nmonte;

	@ generate artificial data @
	ystar = booty(ystart,bhat,ehat,plag,nburn);
	ystar = ystar[nburn+1:rows(ystar),.];    @ throws out the burned observations @

	@ estimate VAR on artificial data with correct number of lags @
	ylag = multlags(ystar,plag,nfirst);
	{bhat1,vhat1,ehat1} = varit(ystar[nfirst:rows(ystar),.],ylag);
	meanb = meanb + reshape(bhat1,ny*(1+ny*plag),1);
	meanb2 = meanb2 + reshape(bhat1^2,ny*(1+ny*plag),1);

 	palt = 12;
	ylag = multlags(ystar,palt,nfirst);
	{bhat2,vhat2,ehat2} = varit(ystar[nfirst:rows(ystar),.],ylag);
	likerat[imonte,1] = nuse*(ln(det(vhat1)) - ln(det(vhat2)));  

imonte = imonte +1;
endo;
"average estimated coefficients";meanis = meanb/nmonte; 
reshape(meanis,ny,1+plag*ny);
"standard errors";
meanis = (meanb2/nmonte) - meanis^2; 
sqrt(reshape(meanis,ny,1+plag*ny));
format /m1;

ljunk = sortc(likerat,1);
ljunk = seqa(1,1,nmonte)~ljunk;  
ljunk[9500:10000,.];


	
	


