@ This GAUSS code controls VAR analysis @
	library pgraph;
	graphset;
	_pdate = 0;
format /m3;

/* ==================================================
	load data
  ==================================================== */
nt = 372;
load y[nt,9] = datajdh.txt;
@ first row 1965:1  last row 1995:12 
 	col (1)  output growth
	col (2)  log price level
	col (3)  log commodity price
	col (4)  Hamilton net oil
	col (5)  fed funds rate
	col (6) short rate
	col (7) long rate  @

/* =======================================================
	set global variables used in procs
    ======================================================= */
nfest = 13;   @ nfest is the first observation used for estimation (after lags) @
nuse = nt - nfest +1;   @ nuse is the number of observations used @
plag = 7;       @ plag is number of lags in VAR @
vhat = 0;         @ this will be set to the structural variance-covariance matrix for VAR @
ks = 48;            @ ks is number of horizons used for forecasts or impulse-response @
ny = 7;            @ ny is number of left-hand variables in VAR @
kc = 1;             @ kc = 2 prints output of VAR analysis, kc = 1 suppresses output @
phi = 0;             @ phi will contain lagged VAR coefficients; set to 0 to load procs @
b0 = 0;

#include varproc2.g;   @ this adds GAUSS VAR procs @

/* =======================================================
	construct data matrix with lags of each variable, 1966:1 to 1995:12 
    ======================================================= */
y2 = makelags(y[.,1],plag,nfest);
i = 2;
do until i > ny;
   y2 = y2 ~ makelags(y[.,i],plag,nfest);
i = i + 1;
endo;

@ delete fed funds rate from explanatory variables for macro block @
y1 = y2[.,1:4*plag]~y2[.,5*plag+1:cols(y2)];

@ eliminate first observations to create dependent vector of length nuse @
y0 = y;
y = y[nfest:nt,.];

/*  =======================================================
	estimate structural VAR
    ======================================================= */
{bhat1,vhat1,ehat1} = varit(y[.,1],ones(nuse,1)~y1);
{bhat2,vhat2,ehat2} = varit(y[.,2],y[.,1]~ones(nuse,1)~y1);
{bhat3,vhat3,ehat3} = varit(y[.,3],y[.,1:2]~ones(nuse,1)~y1);
{bhat4,vhat4,ehat4} = varit(y[.,4],y[.,1:3]~ones(nuse,1)~y1);
{bhat5,vhat5,ehat5} = varit(y[.,5],y[.,1:4]~ones(nuse,1)~y2);
{bhat6,vhat6,ehat6} = varit(y[.,6],y[.,1:5]~ones(nuse,1)~y2);
{bhat7,vhat7,ehat7} = varit(y[.,7],y[.,1:6]~ones(nuse,1)~y2);

@ read contemporaneous coefficients into b0, constant terms into gam0,
	 and delete these from bhat's @
field = 1;
prec = 0;
fmat = "%*.*lf";
b0 = eye(ny);
gam0 = zeros(ny,1);
gam0[1,1] = bhat1[1,1];
bhat1 = bhat1[1,2:cols(bhat1)];
i = 2;
do until i > ny;
   j = 1;
   do until j == i;
	zz = ftos(i,fmat,field,prec);
	qstring = "bhat" $+ zz;
	q = varget(qstring);
 	b0[i,j] = -q[1,j];
   j = j+1;
   endo;
   gam0[i,1] = q[1,i];
   q = q[1,i+1:cols(q)];
   call varput(q,qstring);
i = i+1;
endo;

@ put results together @
bbhat1 = bhat1 | bhat2 | bhat3 | bhat4 ;
bbhat2 = bhat5 | bhat6 | bhat7;
phi =  zeros(ny,cols(bbhat2));
phi[1:4,1:4*plag] = bbhat1[.,1:4*plag];
phi[1:4,5*plag+1:cols(bbhat2)] = bbhat1[.,4*plag+1:cols(bbhat1)];
phi[5:7,.] = bbhat2;


ehat = ehat1 ~ ehat2 ~ ehat3 ~ ehat4 ~ ehat5 ~ ehat6 ~ ehat7 ;
vhat = ehat'*ehat/nuse;

"matrix of contemporaneous coefficients";b0;
"matrix of structural autoregressive coefficients"; phi;
"variance-covariance of structural residuals";vhat;
"standard errors";sqrt(diag(vhat))';
v55 = vhat[5,5];

/* ========================================================
    calculate impulse-response functions
    ======================================================== */
shock = eye(ny);
x0 = zeros(1,ny*plag);
simcont = zeros(ny,1);

ks = 60;
k0 = zeros(ny,ks);
simcont[4,1] = 1;        @ this shuts down feedback of any variables onto oil prices @
k0[.,1] = shock[.,5];    @ this will trace out effect of shock to fed funds rate @
{fall1, eallx} = simul(x0,zeros(ny,ks),k0,ks,simcont);


ks = 48;
k0 = zeros(ny,ks);
simcont = zeros(ny,1);
xtics(0,48, 12, 0);

k0[.,1] = shock[.,4] ;    @ this will trace out effect of shock to oil prices @
{hall1, eall1} = simul(x0,zeros(ny,ks),k0,ks,simcont);



/* ==========================================================
    calculate impulse-response function with fed funds rate constant
    ========================================================== */
k0 = zeros(ny,ks);
k0[.,1] = shock[.,4];

simcont[5,1] = 1;
{hall2,eall2} = simul(x0,zeros(ny,ks),k0,ks,simcont);

ks = 60;
/* ===============================================================
    calculate historical patterns after given shocks 
    ===============================================================  */
@ set date dummies @
topec = 73;
tiran = 157;
tkuwait = 265;
ks = 60;

k0 = gam0*ones(1,ks);
simcont = zeros(ny,1);
simcont[4,1] = 1;

/*   ============= 1973 ========================*/
tnow = topec;
targ = 4;

@ base case @
pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';
{hall720,eall720} = simul(y2[tnow,.],pathy,k0,ks,simcont);

@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall72a,eall72a} = simul(y2[tnow,.],pathy,k0,ks,simcont);

/*   ============= 1979 ========================*/
tnow = tiran;
targ = 9;

@ base case @
simcont = zeros(ny,1);
simcont[4,1] = 1;
pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';
{hall790,eall790} = simul(y2[tnow,.],pathy,k0,ks,simcont);

@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall79a,eall79a} = simul(y2[tnow,.],pathy,k0,ks,simcont);

/*   ============= 1990 ========================*/
tnow = tkuwait;
targ = 7;

@ base case @
simcont = zeros(ny,1);
simcont[4,1] = 1;
pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';
{hall900,eall900} = simul(y2[tnow,.],pathy,k0,ks,simcont);

@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall90a,eall90a} = simul(y2[tnow,.],pathy,k0,ks,simcont);


/* =======================================================
	now redo analysis with 12 lags
    ======================================================= */

plag = 12; 
ks = 48;
k0 = gam0*ones(1,ks);

y = y0;
y2 = makelags(y[.,1],plag,nfest);
i = 2;
do until i > ny;
   y2 = y2 ~ makelags(y[.,i],plag,nfest);
i = i + 1;
endo;


@ delete fed funds rate from explanatory variables for macro block @
y1 = y2[.,1:4*plag]~y2[.,5*plag+1:cols(y2)];

@ eliminate first observations to create dependent vector of length nuse @
y = y[nfest:nt,.];



/*  =======================================================
	estimate structural VAR
    ======================================================= */
{bhat1,vhat1,ehat1} = varit(y[.,1],ones(nuse,1)~y1);
{bhat2,vhat2,ehat2} = varit(y[.,2],y[.,1]~ones(nuse,1)~y1);
{bhat3,vhat3,ehat3} = varit(y[.,3],y[.,1:2]~ones(nuse,1)~y1);
{bhat4,vhat4,ehat4} = varit(y[.,4],y[.,1:3]~ones(nuse,1)~y1);
{bhat5,vhat5,ehat5} = varit(y[.,5],y[.,1:4]~ones(nuse,1)~y2);
{bhat6,vhat6,ehat6} = varit(y[.,6],y[.,1:5]~ones(nuse,1)~y2);
{bhat7,vhat7,ehat7} = varit(y[.,7],y[.,1:6]~ones(nuse,1)~y2);

@ read contemporaneous coefficients into b0, constant terms into gam0,
	 and delete these from bhat's @
field = 1;
prec = 0;
fmat = "%*.*lf";
b0 = eye(ny);
gam0 = zeros(ny,1);
gam0[1,1] = bhat1[1,1];
bhat1 = bhat1[1,2:cols(bhat1)];
i = 2;
do until i > ny;
   j = 1;
   do until j == i;
	zz = ftos(i,fmat,field,prec);
	qstring = "bhat" $+ zz;
	q = varget(qstring);
 	b0[i,j] = -q[1,j];
   j = j+1;
   endo;
   gam0[i,1] = q[1,i];
   q = q[1,i+1:cols(q)];
   call varput(q,qstring);
i = i+1;
endo;

@ put results together @
bbhat1 = bhat1 | bhat2 | bhat3 | bhat4 ;
bbhat2 = bhat5 | bhat6 | bhat7;
phi =  zeros(ny,cols(bbhat2));
phi[1:4,1:4*plag] = bbhat1[.,1:4*plag];
phi[1:4,5*plag+1:cols(bbhat2)] = bbhat1[.,4*plag+1:cols(bbhat1)];
phi[5:7,.] = bbhat2;


ehat = ehat1 ~ ehat2 ~ ehat3 ~ ehat4 ~ ehat5 ~ ehat6 ~ ehat7 ;
vhat = ehat'*ehat/nuse;

/*
/* ========================================================
    calculate impulse-response functions
    ======================================================== */
shock = eye(ny);
x0 = zeros(1,ny*plag);
simcont = zeros(ny,1);

ks = 60;
k0 = zeros(ny,ks);
simcont[4,1] = 1;        @ this shuts down feedback of any variables onto oil prices @
k0[.,1] = shock[.,5];    @ this will trace out effect of shock to fed funds rate @

{fall1, eallx} = simul(x0,zeros(ny,ks),k0,ks,simcont);


ks = 48;
k0 = zeros(ny,ks);
simcont = zeros(ny,1);
xtics(0,48, 12, 0);

k0[.,1] = shock[.,4] ;    @ this will trace out effect of shock to oil prices @
{hall1, eall1} = simul(x0,zeros(ny,ks),k0,ks,simcont);




/* ==========================================================
    calculate impulse-response function with fed funds rate constant
    ========================================================== */
k0 = zeros(ny,ks);
k0[.,1] = shock[.,4];

simcont[5,1] = 1;
{hall2,eall2} = simul(x0,zeros(ny,ks),k0,ks,simcont);



*/



/*   ============= 1973 ========================*/
ks = 60;
k0 = gam0*ones(1,ks);
tnow = topec;
targ = 4;

@ base case @
simcont = zeros(ny,1);
simcont[4,1] = 1;

pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';

{hall720b,eall720} = simul(y2[tnow,.],pathy,k0,ks,simcont);

@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall72ab,eall72a} = simul(y2[tnow,.],pathy,k0,ks,simcont);


/*   ============= 1979 ========================*/
tnow = tiran;
targ = 9;

@ base case @
simcont = zeros(ny,1);
simcont[4,1] = 1;
pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';
{hall790b,eall790} = simul(y2[tnow,.],pathy,k0,ks,simcont);


@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall79ab,eall79a} = simul(y2[tnow,.],pathy,k0,ks,simcont);


/*   ============= 1990 ========================*/
tnow = tkuwait;
targ = 7;

@ base case @
simcont = zeros(ny,1);
simcont[4,1] = 1;
pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';
{hall900b,eall900} = simul(y2[tnow,.],pathy,k0,ks,simcont);


@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall90ab,eall90a} = simul(y2[tnow,.],pathy,k0,ks,simcont);

k0 = gam0*ones(1,ks);

/* ========================================================
    graph policy scenarios  
   ======================================================== */
/* Global Variables for Graphing */
graphset;
axhght = .23;
titht = .4;
axht = .35;

vconst = zeros(ks,1);

"=============================";
begwind;
_pcolor = 12;
_ptek="Fig10.tkf";
axhght = 0.3;
titht = .35;
axht = .35;
xsize = (7.5/2) - .25;
ysize = (6/3) - .25;
makewind(xsize,ysize,1,1+2*ysize,1);
makewind(xsize,ysize,1+xsize,1+2*ysize,1);
makewind(xsize,ysize,1,1+ysize,1);
makewind(xsize,ysize,1+xsize,1+ysize,1);
makewind(xsize,ysize,1,1,1);
makewind(xsize,ysize,1+xsize,1,1);

xxx = seqa(1972,1/12,ks);

fonts("Microb");
setwind(1);
xtics(1972, 1977, 1, 0); 
_pltype =  6 | 6 | 1;
_ptitlht = titht;
_pnumht = axhght;
_plwidth = (7|4|4|4);
_pnum = { 1, 2 ,0};
_paxht = axht;
title("\2011972-77 (7 lags)");
xy(xxx, y[topec:topec+ks-1,1] ~ hall720[1,.]' ~ hall72a[1,.]');

setwind(2);
 graphset;
fonts("Microb");
_pcolor = 12;
_ptek="Fig10.tkf";
xtics(1972, 1977, 1, 0); 
_ptitlht = titht;
_pltype =  6 | 6 | 1 ;
_pnumht = axhght;
_plwidth = (7|4|4|4);
_pnum = { 1, 2 ,0};
_paxht = axht;
title("\2011972-77 (12 lags)");
xy(xxx, y[topec:topec+ks-1,1] ~ hall720b[1,.]' ~ hall72ab[1,.]');

xxx = seqa(1978,1/12,ks);
setwind(3);
xtics(1978, 1983, 1, 0); 
_ptitlht = titht;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
title("\2011978-83 (7 lags)");
xy(xxx, y[tiran:tiran+ks-1,1] ~ hall790[1,.]' ~ hall79a[1,.]');

setwind(4);
xtics(1978, 1983, 1, 0); 
_ptitlht = titht;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
title("\2011978-83 (12 lags)");
xy(xxx, y[tiran:tiran+ks-1,1] ~ hall790b[1,.]' ~ hall79ab[1,.]');

xxx = seqa(1988,1/12,ks);
xtics(1988, 1993, 1, 0);
setwind(5);
_ptitlht = titht;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
title("\2011988-93 (7 lags)");
xy(xxx, y[tkuwait:tkuwait+ks-1,1] ~ hall900[1,.]' ~ hall90a[1,.]');

setwind(6);
xtics(1988, 1993, 1, 0);
_ptitlht = titht;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
title("\2011988-93 (12 lags)");
xy(xxx, y[tkuwait:tkuwait+ks-1,1] ~ hall900b[1,.]' ~ hall90ab[1,.]');
endwind;




