@ This GAUSS code controls VAR analysis @
	library pgraph;
	graphset;
	_pdate = 0;
	output file=junk reset;
	

format /m3;

/* ==================================================
	load data
  ==================================================== */
nt = 372;
load y[nt,9] = datajdh.txt;
@ first row 1965:1  last row 1995:12 
 	col (1)  output growth
	col (2)  log price level
	col (3)  log commodity price
	col (4)  Hamilton net oil
	col (5)  fed funds rate
	col (6) short rate
	col (7) long rate  @

/* =======================================================
	set global variables used in procs
    ======================================================= */
nfest = 13;   @ nfest is the first observation used for estimation (after lags) @
nuse = nt - nfest +1;   @ nuse is the number of observations used @
plag = 7;       @ plag is number of lags in VAR @
vhat = 0;         @ this will be set to the structural variance-covariance matrix for VAR @
ks = 48;            @ ks is number of horizons used for forecasts or impulse-response @
ny = 7;            @ ny is number of left-hand variables in VAR @
kc = 1;             @ kc = 2 prints output of VAR analysis, kc = 1 suppresses output @
phi = 0;             @ phi will contain lagged VAR coefficients; set to 0 to load procs @
b0 = 0;

#include varproc2.g;   @ this adds GAUSS VAR procs @

/* =======================================================
	construct data matrix with lags of each variable, 1966:1 to 1995:12 
    ======================================================= */
y2 = makelags(y[.,1],plag,nfest);
i = 2;
do until i > ny;
   y2 = y2 ~ makelags(y[.,i],plag,nfest);
i = i + 1;
endo;

@ delete fed funds rate from explanatory variables for macro block @
y1 = y2[.,1:4*plag]~y2[.,5*plag+1:cols(y2)];

@ eliminate first observations to create dependent vector of length nuse @
y = y[nfest:nt,.];

/*  =======================================================
	estimate structural VAR
    ======================================================= */
{bhat1,vhat1,ehat1} = varit(y[.,1],ones(nuse,1)~y1);
{bhat2,vhat2,ehat2} = varit(y[.,2],y[.,1]~ones(nuse,1)~y1);
{bhat3,vhat3,ehat3} = varit(y[.,3],y[.,1:2]~ones(nuse,1)~y1);
{bhat4,vhat4,ehat4} = varit(y[.,4],y[.,1:3]~ones(nuse,1)~y1);
{bhat5,vhat5,ehat5} = varit(y[.,5],y[.,1:4]~ones(nuse,1)~y2);
{bhat6,vhat6,ehat6} = varit(y[.,6],y[.,1:5]~ones(nuse,1)~y2);
{bhat7,vhat7,ehat7} = varit(y[.,7],y[.,1:6]~ones(nuse,1)~y2);

@ read contemporaneous coefficients into b0, constant terms into gam0,
	 and delete these from bhat's @
field = 1;
prec = 0;
fmat = "%*.*lf";
b0 = eye(ny);
gam0 = zeros(ny,1);
gam0[1,1] = bhat1[1,1];
bhat1 = bhat1[1,2:cols(bhat1)];
i = 2;
do until i > ny;
   j = 1;
   do until j == i;
	zz = ftos(i,fmat,field,prec);
	qstring = "bhat" $+ zz;
	q = varget(qstring);
 	b0[i,j] = -q[1,j];
   j = j+1;
   endo;
   gam0[i,1] = q[1,i];
   q = q[1,i+1:cols(q)];
   call varput(q,qstring);
i = i+1;
endo;

@ put results together @
bbhat1 = bhat1 | bhat2 | bhat3 | bhat4 ;
bbhat2 = bhat5 | bhat6 | bhat7;
phi =  zeros(ny,cols(bbhat2));
phi[1:4,1:4*plag] = bbhat1[.,1:4*plag];
phi[1:4,5*plag+1:cols(bbhat2)] = bbhat1[.,4*plag+1:cols(bbhat1)];
phi[5:7,.] = bbhat2;


ehat = ehat1 ~ ehat2 ~ ehat3 ~ ehat4 ~ ehat5 ~ ehat6 ~ ehat7 ;
vhat = ehat'*ehat/nuse;

"matrix of contemporaneous coefficients";b0;
"matrix of structural autoregressive coefficients"; phi;
"variance-covariance of structural residuals";vhat;
"standard errors";sqrt(diag(vhat))';
v55 = vhat[5,5];

/* ========================================================
    calculate impulse-response functions
    ======================================================== */
shock = eye(ny);
x0 = zeros(1,ny*plag);
simcont = zeros(ny,1);

ks = 60;
k0 = zeros(ny,ks);
simcont[4,1] = 1;        @ this shuts down feedback of any variables onto oil prices @
k0[.,1] = shock[.,5];    @ this will trace out effect of shock to fed funds rate @
{fall1, eallx} = simul(x0,zeros(ny,ks),k0,ks,simcont);

ks = 48;
k0 = zeros(ny,ks);
simcont = zeros(ny,1);
xtics(0,48, 12, 0);

k0[.,1] = shock[.,4] ;    @ this will trace out effect of shock to oil prices @
{hall1, eall1} = simul(x0,zeros(ny,ks),k0,ks,simcont);


/* ==========================================================
    calculate impulse-response function with fed funds rate constant
    ========================================================== */
k0 = zeros(ny,ks);
k0[.,1] = shock[.,4];

simcont[5,1] = 1;
{hall2,eall2} = simul(x0,zeros(ny,ks),k0,ks,simcont);

"fed funds rate innovations needed to sustain policy";
eall2[5,.];

 @ check that code is correct @
simcont = zeros(ny,1);
k0[5,.] = k0[5,.] + eall2[5,.];
{hallx,eallx} = simul(x0,zeros(ny,ks),k0,ks,simcont);
"following should be zero:"; qh = hall2 - hallx;
qh*qh';   

/* ===============================================================
    graph impulse-response functions 
    =============================================================== */
@ This generates Figure 1 when plag = 7 and Figure 9 when plag = 12 @

proc graphlabel;
if plag == 7;
	_ptek = "Fig1.tkf";
elseif plag == 12;
	_ptek = "Fig9.tkf";
endif;
retp(_ptek);
endp;

ks = 48;
/* Global Variables for Graphing */
axhght = .23;
titht = .4;
axht = .35;

vconst = zeros(ks,1);
begwind;
@ _pmsgstr =
"Figure 1: Impulse-response functions for effect of 10% oil price increase";
_pmsgctl = (.25~7~.5~0~2~15~5);  @
call graphlabel;


@ convert to effect of a 10% oil price increase, simultaneously converting
     logarithms to percent for series 1-4 @
hall1[1:4,.] = hall1[1:4,.]*10;
hall1[5:7,.] = hall1[5:7,.]/10; 
hall2[1:4,.] = hall2[1:4,.]*10;
hall2[5:7,.] = hall2[5:7,.]/10;
eall2[5,.] = eall2[5,.]/10;

xsize = (7.5/2) - .25;
ysize = (6/4) - .25;
makewind(xsize,ysize,1,1+3*ysize,1);
makewind(xsize,ysize,1+xsize,1+3*ysize,1);
makewind(xsize,ysize,1,1+2*ysize,1);
makewind(xsize,ysize,1+xsize,1+2*ysize,1);
makewind(xsize,ysize,1,1+ysize,1);
makewind(xsize,ysize,1+xsize,1+ysize,1);
makewind(xsize,ysize,1,1,1);
makewind(xsize,ysize,1+xsize,1,1);

xxx = seqa(1,1,ks);
z0 = zeros(ks,1);
axhght = 0.4;
titht = .45;
axht = .45;
xtics(0,48,12,0)

setwind(1);
_pcolor=12;
fonts("Microb");
_pltype =  6| 6 | 1 ;
_ptitlht = titht;
_pnumht = axhght;
_plwidth = (4|4|4);
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
if plag==7;
	ytics(-0.25, 0.25, 0.125,0);
elseif plag==12;
	ytics(-0.5,0.25,0.25,0);
endif;
title("\201Output");
xy(xxx, z0~(hall1[1,.] | hall2[1,.])');

setwind(2);
 graphset;
_pcolor=12;
fonts("Microb");
call graphlabel;
xtics(0, 48, 12, 0); 
_ptitlht = titht;
_pltype =  6|6 | 1 ;
_pnumht = axhght;
_plwidth = (4|4|4);
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Prices");
xy(xxx, z0~(hall1[2,.] | hall2[2,.])');

setwind(3);
_ptitlht = titht;
_pltype =  6|6 | 1 ;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Commodity Prices");
xy(xxx, z0~(hall1[3,.] | hall2[3,.])');

setwind(4);
_ptitlht = titht;
_pltype =  6|6 | 1 ;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Oil");
xy(xxx, z0~(hall1[4,.] | hall2[4,.])');

setwind(5);
_ptitlht = titht;
_pltype =  6|6 | 1 ;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Fed Funds");
xy(xxx, z0~(hall1[5,.] | hall2[5,.])');

setwind(6);
_ptitlht = titht;
_pltype =  6|6 | 1 ;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Short Rate");
xy(xxx, z0~(hall1[6,.] | hall2[6,.])');

setwind(7);
_ptitlht = titht;
_pltype =  6|6 | 1 ;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Long Rate");
xy(xxx, z0~(hall1[7,.] | hall2[7,.])');

setwind(8);
_ptitlht = titht;
_pltype =  6 | 1 ;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Policy Innovations");
xy(xxx, z0~eall2[5,.]');

endwind;

ks = 60;

/* ===============================================================
    calculate historical patterns after given shocks 
    ===============================================================  */
@ set date dummies @
topec = 73;
tiran = 157;
tkuwait = 265;
ks = 60;

k0 = gam0*ones(1,ks);
simcont = zeros(ny,1);
simcont[4,1] = 1;

/* @ use these lines for 1973, comment out otherwise @ 
tnow = topec;
targ = 4;
tstart = 1972; */ 

    @ use these lines for 1979, comment out otherwise @
tnow = tiran;
targ = 9;
tstart = 1978;  

/*   @ use these lines for 1990, comment out otherwise @
tnow = tkuwait;
targ = 7;
tstart = 1988;  */

xtics(tstart, tstart+5, 1, 0);

@ base case @
pathy = zeros(ny,ks);
pathy[4,.] = y[tnow:tnow+ks-1,4]';
{hall4,eall4} = simul(y2[tnow,.],pathy,k0,ks,simcont);
"base case forecasts"; hall4; 

@ constant funds rate @
simcont = zeros(ny,1);
simcont[4,1] = 1;
simcont[5,1] = 1;
pathy[5,.] = targ*ones(1,ks);
{hall5,eall5} = simul(y2[tnow,.],pathy,k0,ks,simcont);
 "constant funds rate forecasts";hall5; 

@ xy(seqa(1972,1/12,ks), y2[tnow:tnow+ks-1,1] ~ hall4[1,.]'~hall5[1,.]'); @

 @ check that code is correct @ 
simcont = zeros(ny,1);
simcont[4,1] = 1;
k0[5,.] = k0[5,.] + eall5[5,.];
{hallx,eallx} = simul(y2[tnow,.],pathy,k0,ks,simcont);
"following should be zero:"; qh = hallx - hall5;
qh*qh';   
k0 = gam0*ones(1,ks);

/* ========================================================
    graph policy scenarios for particular episodes
   ======================================================== */
proc graphlabel2;
if tstart == 1972;
	_ptek = "Fig2.tkf";
elseif tstart == 1978;
	_ptek = "Fig4.tkf";
elseif tstart == 1988;
	_ptek ="Fig6.tkf";
endif;
retp(_ptek);
endp;
/* if tstart == 1972;
_pmsgstr =
"Figure 2: Simulating policy for the 1973 oil shock";
elseif tstart == 1978;
_pmsgstr =
"Figure 4: Simulating policy for the 1979 oil shock";
elseif tstart == 1988;
_pmsgstr =
"Figure 6: Simulating policy for the 1990 oil shock";
endif;  */

/* Global Variables for Graphing */
graphset;
_plwidth = (7|4|4|4);
_pcolor=12;
fonts("Microb");
axhght = 0.3;
titht = .35;
axht = .35;

vconst = zeros(ks,1);
begwind;
call graphlabel2;

 _pmsgctl = (3.25~7~.5~0~2~15~5); 
_pltype = (6|6|1);

xsize = (7.5/2) - .25;
ysize = (6/3) - .25;
makewind(xsize,ysize,1,1+2*ysize,1);
makewind(xsize,ysize,1+xsize,1+2*ysize,1);
makewind(xsize,ysize,1,1+ysize,1);
makewind(xsize,ysize,1+xsize,1+ysize,1);
makewind(xsize,ysize,1,1,1);
makewind(xsize,ysize,1+xsize,1,1);


xxx = seqa(tstart,1/12,ks);
xtics(tstart, tstart+5, 1, 0); 

setwind(1);
_pltype =  6 | 6 | 1;
_ptitlht = titht;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Logarithmic growth");
title("\201Output");
xy(xxx, y[tnow:tnow+ks-1,1] ~ hall4[1,.]' ~ hall5[1,.]');

setwind(2);
 graphset;
_plwidth = (7|4|4|4);
_pcolor=12;
fonts("Microb");
call graphlabel2;
xtics(tstart, tstart+5, 1, 0); 
_ptitlht = titht;
_pltype =  6 | 6 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Logarithm");
title("\201Prices");
xy(xxx, y[tnow:tnow+ks-1,2] ~ hall4[2,.]' ~ hall5[2,.]');

setwind(3);
_ptitlht = titht;
_pltype =  6 | 6 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Logarithm");
title("\201Commodity Prices");
xy(xxx, y[tnow:tnow+ks-1,3] ~ hall4[3,.]' ~ hall5[3,.]');

setwind(4);
_ptitlht = titht;
_pltype =  6 | 6 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Fed Funds");
xy(xxx, y[tnow:tnow+ks-1,5] ~ hall4[5,.]' ~ hall5[5,.]');

setwind(5);
_ptitlht = titht;
_pltype =  6 | 6 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Short Rate");
xy(xxx, y[tnow:tnow+ks-1,6] ~ hall4[6,.]' ~ hall5[6,.]');

setwind(6);
_ptitlht = titht;
_pltype =  6 | 6 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Long Rate");
xy(xxx, y[tnow:tnow+ks-1,7] ~ hall4[7,.]' ~ hall5[7,.]');


endwind;

/* ========================================================
    calculate cumulative consequences of implied fed funds shocks and check 
    ======================================================== */

theo = zeros(ny,ks);      @ theo will contain the theoretical standard error for effect of shock at
					    horizon ks @

v1 = v55*fall1[.,1] * fall1[.,1]';
val = zeros(ny,ks);
val[.,1] = fall1[.,1] *eall5[5,1];
theo[.,1] = 2*sqrt(diag(v1));

i = 2;
v = v1;
do until i > ks;
	v = v + v55*fall1[.,i]*fall1[.,i]';
	j = 1;
	do until j > i;
		val[.,i] = val[.,i] + fall1[.,j]*eall5[5,i-j+1];
	j = j+1;
	endo;
	zz = ftos(i,fmat,field,prec);
	vstring = "v" $+ zz;
	call varput(v,vstring);
	theo[.,i]=	2*sqrt(diag(v));
i = i+1;
endo;

@ try to replicate with bootstrapping @
nehat = rows(ehat5);
seed1 = 3937841;
estar = zeros(1,ks);  @ this will contain the shocks @

nmont = 100;   @ final output used 10000 @
iy = 1;
do until iy > ny;
zz = ftos(iy,fmat,field,prec);
vstring = "effstar" $+ zz;
v = zeros(nmont,ks);
call varput(v,vstring);
iy = iy +1;
endo;

imonte = 1;
do until imonte > nmont;
	ustar = 1+trunc(nehat*rndus(1,ks,seed1));   
		@ this generates a random vector, each of whose elements is equally likely
			 to be one of the integers {1,2,...,nehat} @
	j = 1;
	do until j > ks;
		estar[1,j] = ehat5[ustar[1,j],1];    @ this draws a shock for horizon j @
	j = j+1;
	endo;
		v = fall1.*estar;
		@ fall1 is an (ny x ks) matrix
		     estar is a (1 x ks) vector
		     hence v is (ny x ks)  @
	@ cumulate consequences of shocks @
	j = 2;
	do until j > ks;
		v[.,j] = v[.,j-1] + v[.,j];
		j = j+1;
	endo;
	@ put summary of shocks for variable i in effstari @
		effstar1[imonte,.] = v[1,.];
		effstar2[imonte,.] = v[2,.];
		effstar3[imonte,.] = v[3,.];
		effstar4[imonte,.] = v[4,.];
		effstar5[imonte,.] = v[5,.];
		effstar6[imonte,.] = v[6,.];
		effstar7[imonte,.] = v[7,.];

imonte = imonte +1;
endo;


@ report 95% confidence intervals for each variable @
sig = .025;  @ change to .025 once code is working @
qlow = zeros(ny,ks);
qhigh = zeros(ny,ks);
iy = 1;
do until iy > ny;
	zz = ftos(iy,fmat,field,prec);
	vstring = "effstar" $+ zz;
	q = varget(vstring);
	j = 1;
	do until j > ks;
		q0 = q[.,j];
		q0 = sortc(q0,1);
		qlow[iy,j] = q0[1+trunc(sig*nmont),1]; 
		qhigh[iy,j] = q0[1+trunc((1 - sig)*nmont),1];
	j = j+1;
	endo;
iy = iy + 1;
endo;

"==============================";
format /m1;
iy = 1;
do until iy > ny;
	"";"";"Variable";;iy;
	junk = theo[iy,.]' ~ qlow[iy,.]' ~ qhigh[iy,.]'; junk;
	iy = iy+1;
endo;
format /m3;
	
qh = hall5 - hall4 - val;
"check: should be zero";
qh*qh'; 

vwhole = zeros(12,ks);

i = 1;                @ i indexes variable i @
jv= 1;                 @ j indexes rows of vwhole, to reconstruct values for graphing @

do until i > ny;

labels = "Output" $| "Prices" $| "Commodity Prices" $| "Oil" ;
labels = labels $| "Fed Funds" $| "Short Rate" $| "Long Rate";
xtics(1972, 1977, 1, 0);

@ read standard errors into vector v @
v = zeros(1,ks);  
j = 1;
do until j > ks;
	zz = ftos(j,fmat,field,prec);
	vstring = "v" $+ zz;
	q = varget(vstring);
	v[1,j] = q[i,i];
j = j+1;
endo;
v = 2*sqrt(v);
if i > 4;
	v = -v;
endif;

vwhole[jv,.] = val[i,.];
vwhole[jv+1,.] = v;
jv = jv+2;

title(labels[i,1]);

if i == 3;
	i = i+2;
else;
	i = i+1;
endif;
endo;

/********************************************************************
* GRAPH POLICY PLAUSIBILITY CALCULATIONS
********************************************************************/
proc graphlabel3;
if tstart == 1972;
	_ptek = "Fig3.tkf";
elseif tstart == 1978;
	_ptek = "Fig5.tkf";
elseif tstart == 1988;
	_ptek ="Fig7.tkf";
endif;
retp(_ptek);
endp;

/* if tstart == 1972;
_pmsgstr =
"Figure 3: Implied policy effects and 95% confidence intervals for 1972-77";
elseif tstart == 1978;
_pmsgstr =
"Figure 5: Implied policy effects and 95% confidence intervals for 1978-83";
elseif tstart == 1988;
_pmsgstr =
"Figure 7: Implied policy effects and 95% confidence intervals for 1988-93";
endif;  */



/* Global Variables for Graphing */
vconst = zeros(ks,1);
 _pmsgctl = (.25~7~.5~0~2~15~5); 


begwind;
call graphlabel3;
_pltype = (6|1|6|6);
axhght = 0.3;
titht = .35;
axht = .35;
xsize = (7.5/2) - .25;
ysize = (6/3) - .25;
makewind(xsize,ysize,1,1+2*ysize,1);
makewind(xsize,ysize,1+xsize,1+2*ysize,1);
makewind(xsize,ysize,1,1+ysize,1);
makewind(xsize,ysize,1+xsize,1+ysize,1);
makewind(xsize,ysize,1,1,1);
makewind(xsize,ysize,1+xsize,1,1);

xxx = seqa(tstart,1/12,ks);

setwind(1);
xtics(tstart, tstart+5, 1, 0); 
_pltype =  6 | 1 | 1 | 2;
_ptitlht = titht;
_pnumht = axhght;
_plwidth = (4|4|4|4);
_pcolor=12;
fonts("Microb");
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Logarithmic growth");
title("\201Output");
xy(xxx, vwhole[1:2,.]'~-vwhole[2,.]'~vconst);

setwind(2);
 graphset;
call graphlabel3;
xtics(tstart, tstart+5, 1, 0); 
_ptitlht = titht;
_pltype =  6 | 1 | 1 | 2;
_pnumht = axhght;
_plwidth = (4|4|4|4);
_pcolor=12;
fonts("Microb");
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Logarithm");
title("\201Prices");
xy(xxx, vwhole[3:4,.]'~-vwhole[4,.]'~vconst);

setwind(3);
_ptitlht = titht;
_pltype =  6 | 1 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Logarithm");
title("\201Commodity Prices");
xy(xxx, vwhole[5:6,.]'~-vwhole[6,.]'~vconst);

setwind(4);
_ptitlht = titht;
_pltype =  6 | 1 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Fed Funds");
xy(xxx, vwhole[7:8,.]'~-vwhole[8,.]'~vconst);

setwind(5);
_ptitlht = titht;
_pltype =  6 | 1 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Short Rate");
xy(xxx, vwhole[9:10,.]'~-vwhole[10,.]'~vconst);

setwind(6);
_ptitlht = titht;
_pltype =  6 | 1 | 1 | 2;
_pnumht = axhght;
_pnum = { 1, 2 ,0};
_paxht = axht;
ylabel("Percent");
title("\201Long Rate");
xy(xxx, vwhole[11:12,.]'~-vwhole[12,.]'~vconst);

endwind;


